﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 15 $
Author(s): Myrroddin of Llane (Alliance US)
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, AceComm-3.0, AceSerializer-3.0, LibCompress
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 15 $"):match("%d+"))

if not LibStub then error(MAJOR_VERSION.."requires LibStub") end
local lib, oldversion = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local _G = getfenv(0)

LibStub("AceSerializer-3.0"):Embed(lib)
LibStub("AceComm-3.0"):Embed(lib)
local CallbackHandler = LibStub("CallbackHandler-1.0")
local LibCompress = LibStub("LibCompress")

lib.eventFrame = lib.eventFrame or CreateFrame("Frame")
lib.eventFrame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
lib.eventFrame:UnregisterAllEvents()

lib.guildBank = lib.guildBank or {}
local oldRank
local guildRank
local joinedGuild
local playerRankChanged
local logged_in

lib.callbacks = lib.callbacks or CallbackHandler:New(lib)

local recHash -- received hash
function lib:OnCommReceived(prefix, msg, distribution, sender)
	if prefix ~= "LGBC" then return end
    if sender == UnitName("player") then return end
	
	local command, value = msg:match("^(%S+) (.*)$")
				
	if cmd == "REQUEST" then
		-- serialize lib.guildBank into a string
		-- "zip up" the serialized string
		-- create a hash for comparison
		-- send the hash with higher prio than the serialized string and the serialized string			
		local serialized_data, serialization_error = lib:Serialize(lib.guildBank)
		if serialization_error then
			return print(MAJOR_VERSION.."Error during serialization:", serialization_error)
		end
			
		local compressed_data, compression_error = LibCompress:Compress(serialized_data)
		if compression_error then
			return print(MAJOR_VERSION.."Error during compression:", compression_error)
		end
			
		local hash = LibCompress:fcs32init()
		hash = LibCompress:fcs32update(hash, serialized_data)
		hash = LibCompress:fcs32final(hash)
			
		-- Construct a comm string for the hash.
		local commHash = "HASH "..hash
		lib:SendCommMessage("LGBC", commHash, "WHISPER", sender, "ALERT")
			
		-- Construct a comm string for the compressed data.
		local commString = "UPDATE "..compressed_data
		lib:SendCommMessage("LGBC", commString, "WHISPER", sender, "BULK")
		--@debug
		print("Sending bank info to"..sender)
		--@end-debug			
	elseif cmd == "UPDATE" then
		-- create a new hash
		local compare_hash = LibCompress:fcs32init()
		compare_hash = LibCompress:fcs32upate(compare_hash, value)
		compare_hash = LibCompress:fcs32final(compare_hash)
			
		-- compare hashes
		-- if no match then deserialize, decompress, check for errors, and update
		-- only update lib.guildBank locally if no errors
		if compare_hash ~= recHash then
			local decompressed_data, decompression_error = LibCompress:Decompress(value)
			if decompression_error then
				return print(MAJOR_VERSION.." Error during decompression:", decompression_error)
			end
				
			local deserialized_data, deserialization_error = lib:Deserialize(decompressed_data)			
			if deserialization_error then
				return print(MAJOR_VERSION.." Error during deserialization:", deserialization_error)
			else				
				lib.guildBank = deserialized_data
				lib:FilterPages()
			end
		--[===[@debug@
		else
			return print("Received hash matches generated hash. Not updating.")
		--@end-debug@]===]
		end
	elseif cmd == "HASH" then
		recHash = value
	end
end

lib:RegisterComm("LGBC")

function lib:FilterPages()
	local guildName = GetGuildInfo("player")
	local pName = UnitName("player")
	-- _G.GuildControlSetRank(guildRank)
	for page = 1, lib.guildBank.numTabs do
		local canView = GetGuildBankTabPermissions(page)
		if not canView then
			lib.guildBank[page].links = nil -- if the player can no longer see a page, nil the info
			lib.guildBank[page].stacks = nil
		end
	end
	-- update addon
	local bankData = lib.guildBank
	lib.callbacks:Fire("GuildBankComm_ContentsChanged", pName, bankData, guildName)
	--[===[@debug@
	print(pName.." from "..guildName.." has fired GuildBankComm_ContentsChanged")
	--@end-debug@]===]	
end

function lib:RequestDataAboutPlayer()
	local _, _, rankIndex = GetGuildInfo("player")
	guildRank = rankIndex
	--[===[@debug@
	print("Player's guild rank is "..guildRank)
	--@end-debug@]===]
	if (oldRank ~= guildRank) then
		lib:SendCommMessage("LGBC", "REQUEST", "GUILD", nil, "BULK")
		oldRank = guildRank	
		lib:FilterPages()
	end
end

function lib.eventFrame:GUILD_ROSTER_UPDATE(unit)
	-- if the player joins a guild
	-- or if the player is in a guild at PLAYER_LOGIN
	-- or is the player's rank changed
	-- we need to know about the player's rank privileges
	
	if unit ~= UnitName("player") then return end
	
	if (IsInGuild() and joinedGuild) or (playerRankChanged) or (logged_in) then
		lib:RequestDataAboutPlayer()
	end
	-- logged_in should be true only for PLAYER_LOGIN() and /reloadUI
	-- GUILD_ROSTER_UPDATE() and GUILD_RANKS_UPDATE() fire at same time when a unit's rank changes
	-- therefore reset playerRankChange in case player's rank changes rapidly and often
	playerRankChanged = false
	logged_in = false
end

function lib.eventFrame:GUILD_RANKS_UPDATE(unit)
	if unit ~= UnitName("player") then return end
	playerRankChanged = true
end

function lib.eventFrame:PLAYER_GUILD_UPDATE()
	if not IsInGuild() then -- player left guild			
		local pName = UnitName("player")
		wipe(lib.guildBank)
		local bankData = lib.guildBank
		lib.callbacks:Fire("GuildBankComm_PlayerGuildless", pName, bankData)
		joinedGuild = false			
		-- guild leaders and unguilded people both have rank 0 just to be confusing
		-- therefore we need to force the issue when leaving a guild
		oldRank = 100 -- set to impossible rank number
		guildRank = 0
		playerRankChanged = false
	else
		joinedGuild = true
	end
end

function lib.eventFrame:GUILDBANKFRAME_OPENED()	
	lib.guildBank.numTabs = GetNumGuildBankTabs()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	local tab = GetCurrentGuildBankTab()
	print("Guild Bank frame open. Looking at tab "..tostring(tab))
	--@end-debug@]===]
	
	local name, icon
	for page = 1, lib.guildBank.numTabs do
		name, icon = GetGuildBankTabInfo(page)
		
		if not lib.guildBank[page] then
			lib.guildBank[page] = {
				links = {},  --store the slot item links here
				stacks = {}, --store the slot stack counts here
				name = name,
				texture = icon
			}
		end
	end
end

function lib.eventFrame:GUILDBANK_UPDATE_MONEY()
	lib.guildBank.guildFunds = GetGuildBankMoney()
	--[===[@debug@
	local guildName = GetGuildInfo("player")
	print(guildName.." has "..lib.guildBank.guildFunds.." in copper")
	--@end-debug@]===]
end

function lib.eventFrame:GUILDBANK_UPDATE_TABS()
	lib.guildBank.numTabs = GetNumGuildBankTabs()
end

local num, itemString
function lib.eventFrame:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[page].links[slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[page].stacks[slot] = num
			else
				-- that slot is empty
				lib.guildBank[page].links[slot] = nil
				lib.guildBank[page].stacks[slot] = nil
			end
			num, itemString = nil, nil
		end
		-- time stamp
		lib.guildBank[page].lastScan = time()
	end
end

function lib.eventFrame:GUILDBANKFRAME_CLOSED()
	-- we want to filter what the player sees
	-- and only send that rather than the whole of lib.guildBank
	lib:FilterPages()
	local serialized_data, serialization_error
	local compressed_data, compression_error
	local hash
	
	-- Attempt to serialize the data.
	serialized_data, serialization_error = lib:Serialize(lib.guildBank)

	-- If serialization failed, spit out a message explaining why, and don't try to continue.
	if serialization_error then
		return print(MAJOR_VERSION.."Error during serialization:", serialization_error)
	end

	-- Compress the serialized data.
	compressed_data, compression_error = LibCompress:Compress(serialized_data)
	if compression_error then
		return print (MAJOR_VERSION.."Error during compression:", compression_error)
	end

	-- Get a hash for the serialized data.
	hash = LibCompress:fcs32init()
	hash = LibCompress:fcs32update(hash, serialized_data)
	hash = LibCompress:fcs32final(hash)

	-- Construct a comm string for the hash.
	local commHash = "HASH "..hash

	-- Do stuff with the hash comm string.
	--[===[@debug@
	print("commHash has a length of", commHash:len())
	--@end-debug@]===]

	lib:SendCommMessage("LGBC", commHash, "GUILD", nil, "ALERT")

	-- Construct a comm string for the compressed data.
	local commString = "UPDATE "..compressed_data
		
	-- Do stuff with the data comm string.
	--[===[@debug@
	print("commString has a length of", commString:len())
	--@end-debug@]===]
	lib:SendCommMessage("LGBC", commString, "GUILD", nil, "BULK")
end

function lib.eventFrame:PLAYER_LOGIN()	
	oldRank = 100 -- set to impossible rank number
		logged_in = true
	--[===[@debug@
	print("PLAYER_LOGIN fired. Old rank:"..oldRank)
	--@end-debug@]===]
	if IsInGuild() then
		_G.GuildRoster()
	end
end

function lib:Start()
	lib.eventFrame:RegisterEvent("PLAYER_GUILD_UPDATE")
	lib.eventFrame:RegisterEvent("GUILD_RANKS_UPDATE")
	lib.eventFrame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_OPENED")
	lib.eventFrame:RegisterEvent("GUILDBANKFRAME_CLOSED")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	lib.eventFrame:RegisterEvent("GUILDBANK_UPDATE_MONEY")	
	lib.eventFrame:RegisterEvent("PLAYER_LOGIN")
	lib.eventFrame:RegisterEvent("GUILD_ROSTER_UPDATE")
end
lib:Start()